/*
 * Scratchpad MMU - Serial line boot loader
 * Copyright (C) Jack Whitham 2009
 * http://www.jwhitham.org.uk/c/smmu.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation
 * (version 2.1 of the License only).
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "xparameters.h"

/* Frequency setting for UART divisor */
#define BAUD 115200
#define FREQ ((XPAR_UARTNS550_0_CLOCK_FREQ_HZ + (BAUD * 8)) / (BAUD * 16))

#include "stdio.h"

#include "xuartns550_l.h"
#include "xutil.h"

#define AGEN(a) \
    (* ( (volatile char *) ( (unsigned) XPAR_RS232_UART_1_BASEADDR + \
            (unsigned) (a) )))
#define UWRITE(a, d) AGEN((a)) = (d) 
#define UREAD(a) AGEN(a)

typedef void (* Target_Fn) ( void ) ;

static void printchar ( char s )
{
    while ( ! ( UREAD(XUN_LSR_OFFSET) & XUN_LSR_TX_BUFFER_EMPTY ) )
    {
    }
    UWRITE(XUN_THR_OFFSET, s);
}

static unsigned getbyte ( void )
{
    while ( ! ( UREAD(XUN_LSR_OFFSET) & XUN_LSR_DATA_READY ))
    {
    }
    return (unsigned) ( (unsigned char) UREAD(XUN_RBR_OFFSET) ) ;
}

int main ( void )
{
    int             i ;
    unsigned        size = 0 ;
    unsigned        target = 0 ;
	const unsigned  LcrRegister = XUN_LCR_8_DATA_BITS;

	UWRITE(XUN_LCR_OFFSET, LcrRegister | XUN_LCR_DLAB);

	/*
	 * Set the baud Divisors to set rate, the initial write of 0xFF is
	 * to keep the divisor from being 0 which is not recommended as per
	 * the NS16550D spec sheet
	 */
	UWRITE(XUN_DRLS_OFFSET, 0xFF);
	UWRITE(XUN_DRLM_OFFSET, 0);
	UWRITE(XUN_DRLS_OFFSET, FREQ);
   

	/*
	 * Clear the Divisor latch access bit, DLAB to allow nornal
	 * operation and write to the line control register
	 */
	UWRITE(XUN_LCR_OFFSET, LcrRegister);

    printchar ( 'h' ) ;
    printchar ( 'i' ) ;
    for ( i = 0 ; i < 4 ; i ++ )
    {
        size = ( size << 8 ) | (unsigned) ( (unsigned char) getbyte () ) ;
    }

    size = size >> 2 ;

    printchar ( 'l' ) ;
    target = 0 ;
    for ( i = 0 ; i < 4 ; i ++ )
    {
        target = ( target << 8 ) | (unsigned) ( (unsigned char) getbyte () ) ;
    }

    {
        volatile unsigned * target_data = (unsigned *) target ;
        volatile unsigned char * td2 = (unsigned char *) target_data ;
        Target_Fn       target_code ;

        td2 [ 0 ] = 0x12 ;
        td2 [ 1 ] = 0x34 ;
        if ( td2 [ 0 ] != 0x12 )
        {
            printchar ( 'X' ) ;
        }
        if ( td2 [ 1 ] != 0x34 )
        {
            printchar ( 'Y' ) ;
        }
        target_code = (Target_Fn) target_data ;
        printchar ( 'k' ) ;

        for ( i = 0 ; i < size ; i ++ )
        {
            unsigned r = ( getbyte () << 24 ) |
                    ( getbyte () << 16 ) |
                    ( getbyte () << 8 ) |
                    ( getbyte () << 0 ) ;

            target_data [ i ] = r ;
            target_data [ i + 1 ] = 0 ;

            if ( target_data [ i ] != r )
            {
                printchar ( '?' ) ;
                while ( 1 ) {}
            } else if (( i & 0x3ff ) == 0 )
            {
                printchar ( '.' ) ;
            }
        }
        printchar ( 'g' ) ;
        target_code () ;
        printchar ( '?' ) ;
    }
    return 0 ;
}

